@@ -14,10 +14,10 @@
 # limitations under the License.
 
 try:
-    __version__ = __import__('pkg_resources').get_distribution('simplecouchdb').version
+    __version__ = __import__('pkg_resources').get_distribution('py-simplecouchdb').version
 except:
     __version__ = '?'
 
 
-from simplecouchdb.client import Server
+from simplecouchdb.core import Server, Database
 
