@@ -1,32 +1,32 @@
-# -*- coding: utf-8 -
-# Copyright (c) 2008, 2009 Benoît Chesneau <benoitc@e-engura.com>
+# -*- coding: utf-8 -*-
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Copyright (c) 2008-2009 Benoit Chesneau <benoitc@e-engura.com> 
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+# Permission to use, copy, modify, and distribute this software for any
+# purpose with or without fee is hereby granted, provided that the above
+# copyright notice and this permission notice appear in all copies.
+#
+# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 
 from calendar import timegm
 import decimal
 import datetime
 import time
 
-from simplecouchdb.schema.properties.view import *
+from simplecouchdb.schema.exceptions import *
 
-class DuplicatePropertyError(Exception):
-    """ exception raised when there is a duplicate 
-    property in a model """
 
-class BadValueError(Exception):
-    """ exception raised when a value can't be validated 
-    or is required """
+__all__ = ['Property', 'StringProperty', 'IntegerProperty',
+        'DecimalProperty', 'BooleanProperty', 'FloatProperty',
+        'DateTimeProperty', 'DateProperty', 'TimeProperty',
+        'ListProperty', 'DictProperty']
 
 class Property(object):
     """ Propertu base which all other properties
@@ -60,6 +60,13 @@ class Property(object):
         if self.name is None:
             self.name = property_name
 
+    def __property_init__(self, document_instance, value):
+        """ method used to set value of the property when
+        we create the document. Don't check required. """
+        if value is not None:
+            value = self._to_json(self.validate(value, required=False))
+        document_instance._doc[self.name] = value
+
     def __get__(self, document_instance, document_class):
         if document_instance is None:
             return self
@@ -67,12 +74,12 @@ class Property(object):
         value = document_instance._doc.get(self.name)
         if value is not None:
             value = self._to_python(value)
+        
         return value
 
     def __set__(self, document_instance, value):
-        if value is not None:
-            value = self._to_json(self.validate(value))
-        document_instance._doc[self.name] = value
+        value = self.validate(value, required=False)
+        document_instance._doc[self.name] = self._to_json(value)
 
     def __delete__(self, document_instance):
         pass
@@ -85,12 +92,11 @@ class Property(object):
             default = default()
         return default
 
-    def validate(self, value):
+    def validate(self, value, required=True):
         """ validate value """
-        if self.empty(value):
+        if required and self.empty(value):
             if self.required:
-                raise BadValueError('Property %s is required' 
-                        % self.name)
+                raise BadValueError("Property %s is required." % self.name)
         else:
             if self.choices:
                 match = False
@@ -98,7 +104,8 @@ class Property(object):
                     if choice == value:
                         match = True
                     if not match:
-                        raise BadValueError('Property %s is %r; must be one of %r' % (self.name, value, self.choices))
+                        raise BadValueError('Property %s is %r; must be one of %r' % (
+                            self.name, value, self.choices))
         if self.validators:
             if isinstance(self.validators, (list, tuple,)):
                 for validator in self.validators:
@@ -128,14 +135,19 @@ class StringProperty(Property):
     
     *Value type*: unicode
     """
-    _to_python = unicode
 
-    def validate(self, value):
-        value = super(StringProperty, self).validate(value)
-        if value is not None and not isinstance(value, basestring):
+    _to_python = unicode 
+
+    def validate(self, value, required=True):
+        value = super(StringProperty, self).validate(value,
+                required=required)
+
+        if value is None:
+            return value
+        
+        if not isinstance(value, basestring):
             raise BadValueError(
                 'Property %s must be unicode or str instance, not a %s' % (self.name, type(value).__name__))
-
         return value
 
     data_type = unicode
@@ -147,32 +159,27 @@ class IntegerProperty(Property):
     """
     _to_python = int
 
-    def validate(self, value):
-        value = super(IntegerProperty, self).validate(value)
-        if value is not None and not isinstance(value, int):
+    def empty(self, value):
+        return value is None
+
+    def validate(self, value, required=True):
+
+        value = super(IntegerProperty, self).validate(value,
+                required=required)
+
+        if value is None:
+            return value
+
+        if value is not None and not isinstance(value, (int, long,)):
             raise BadValueError(
-                'Property %s must be int or long instance, not a %s'
-                % (self.name, type(value).__name__)) 
-        return value
+                'Property %s must be %s or long instance, not a %s'
+                % (self.name, type(self.data_type).__name__, 
+                    type(value).__name__)) 
+
+        return value 
 
     data_type = int
-
-class LongProperty(Property):
-    """ Long property, map to python long.
-    
-    *Value type*: long
-    """
-    _to_python = long
-
-    def validate(self, value):
-        value = super(LongProperty, self).validate(value)
-        if value is not None and not isinstance(value, long):
-            raise BadValueError(
-                'Property %s must be int or long instance, not a %s'
-                % (self.name, type(value).__name__)) 
-        return value
-
-    data_type = long
+LongProperty = IntegerProperty
 
 class FloatProperty(Property):
     """ Float property, map to python float 
@@ -182,13 +189,18 @@ class FloatProperty(Property):
     _to_python = float
     data_type = float
 
-    def validate(self, value):
-        value = super(FloatProperty, self).validate(value)
-        if value is not None and not isinstance(value, float):
+    def validate(self, value, required=True):
+        value = super(FloatProperty, self).validate(value, 
+                required=required)
+
+        if value is None:
+            return value
+
+        if not isinstance(value, float):
             raise BadValueError(
                 'Property %s must be float instance, not a %s'
                 % (self.name, type(value).__name__))
-
+        
         return value
 Number = FloatProperty
 
@@ -200,13 +212,18 @@ class BooleanProperty(Property):
     _to_python = bool
     data_type = bool
 
-    def validate(self, value):
-        value = super(BooleanProperty, self).validate(value)
+    def validate(self, value, required=True):
+        value = super(BooleanProperty, self).validate(value, 
+                required=required)
+
+        if value is None:
+            return value
+
         if value is not None and not isinstance(value, bool):
             raise BadValueError(
                 'Property %s must be bool instance, not a %s'
                 % (self.name, type(value).__name__))
-
+        
         return value
 
 class DecimalProperty(Property):
@@ -236,11 +253,15 @@ class DateTimeProperty(Property):
         self.auto_now = auto_now
         self.auto_now_add = auto_now_add
 
-    def validate(self, value):
-        value = super(DateTimeProperty, self).validate(value)
+    def validate(self, value, required=True):
+        value = super(DateTimeProperty, self).validate(value, required=required)
+
+        if value is None:
+            return value
+
         if value and not isinstance(value, self.data_type):
-            raise BadValueError('Property %s must be a %s' %
-                          (self.name, self.data_type.__name__))
+            raise BadValueError('Property %s must be a %s, current is %s' %
+                          (self.name, self.data_type.__name__, type(value).__name__))
         return value
 
     def default_value(self):
@@ -324,13 +345,54 @@ class ListProperty(Property):
     *ValueType*: list
     """
 
-    _to_python = list
+    data_type = list
 
-    def validate(self, value):
-        value = super(ListProperty, self).validate(value)
+    def validate(self, value, required=True):
+        value = super(ListProperty, self).validate(value, 
+                required=required)
 
-        if not isinstance(value, (list, tuple,)):
+        if value is None:
+            return value
+
+        if not isinstance(value, list):
             raise BadValueError('Property %s must be a list' %
                     self.name)
-
         return value
+
+    def default_value(self):
+        value = super(ListProperty, self).default_value()
+        if value is None:
+            value = []
+        return list()
+
+    def _to_python(self, value):
+        return value 
+
+class DictProperty(Property):
+    """Dict property. Map to python dict
+    
+    *ValueType*: dict
+    """
+    data_type = dict
+
+    def validate(self, value, required=True):
+        value = super(DictProperty, self).validate(value, 
+                required=required)
+
+        if value is None:
+            return value
+
+        if not isinstance(value, dict):
+            raise BadValueError('Property %s must be a dict, current is' %
+                    self.name, type(value).__name__)
+        return value
+
+    def default_value(self):
+        value = super(DictProperty, self).default_value() 
+        if value is None:
+            value = {}
+        return dict(value)
+
+    def _to_python(self, value):
+        return value
+
